==> flutter_snow_base-2.1.0.tar.gz: lib/src/widgets/list_view/list_view_resource_widget.dart <==
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart' hide ReorderableList;
import 'package:flutter_snow_base/flutter_snow_base.dart';
import 'package:flutter_snow_base/src/widgets/error_snow/error_snow_widget.dart';
import 'package:flutter_reorderable_list/flutter_reorderable_list.dart';

class ListViewResourceWidget<T> extends StatelessWidget {
  final bool useSliver;
  final Resource<List<T>> resource;
  final Widget loadingTile;
  final Widget Function(T data) tileMapper;
  final Widget Function(int index, Widget tile)? separatorBuilder;
  final Widget? emptyWidget;
  final Future<void> Function()? refresh;
  final EdgeInsets? padding;
  final List<Widget> topWidgets;
  final Widget Function(AppException e)? errorWidget;
  final Axis scrollDirection;
  final bool reverse;
  final ScrollController? controller;
  final bool? primary;
  final ScrollPhysics? physics;
  final bool shrinkWrap;
  final bool addAutomaticKeepAlives;
  final bool addRepaintBoundaries;
  final bool addSemanticIndexes;
  final double? cacheExtent;
  final int? semanticChildCount;
  final DragStartBehavior dragStartBehavior;
  final ScrollViewKeyboardDismissBehavior keyboardDismissBehavior;
  final String? restorationId;
  final Clip clipBehavior;
  final double? itemExtent;
  final bool reorderableList;
  final ReorderCallback? onReorder;
  final int loadingTileQuantity;

  const ListViewResourceWidget({
    Key? key,
    this.useSliver = false,
    required this.resource,
    required this.loadingTile,
    required this.tileMapper,
    this.refresh,
    this.emptyWidget,
    this.padding,
    this.topWidgets = const [],
    this.errorWidget,
    this.scrollDirection = Axis.vertical,
    this.reverse = false,
    this.controller,
    this.primary,
    this.physics,
    this.shrinkWrap = true,
    this.itemExtent,
    this.addAutomaticKeepAlives = true,
    this.addRepaintBoundaries = true,
    this.addSemanticIndexes = true,
    this.cacheExtent,
    this.semanticChildCount,
    this.dragStartBehavior = DragStartBehavior.start,
    this.keyboardDismissBehavior = ScrollViewKeyboardDismissBehavior.manual,
    this.restorationId,
    this.clipBehavior = Clip.hardEdge,
    this.loadingTileQuantity = 2,
    this.separatorBuilder,
  })  : reorderableList = false,
        onReorder = null,
        super(key: key);

  const ListViewResourceWidget.reorderable({
    Key? key,
    required this.resource,
    required this.loadingTile,
    required this.tileMapper,
    required this.onReorder,
    this.refresh,
    this.emptyWidget,
    this.topWidgets = const [],
    this.errorWidget,
    this.padding,
    this.controller,
    this.scrollDirection = Axis.vertical,
    this.shrinkWrap = true,
    this.loadingTileQuantity = 2,
    this.separatorBuilder,
    this.physics,
    this.reverse = false,
    this.keyboardDismissBehavior = ScrollViewKeyboardDismissBehavior.manual,
    this.dragStartBehavior = DragStartBehavior.start,
    this.restorationId,
    this.cacheExtent,
    this.clipBehavior = Clip.hardEdge,
    this.primary,
  })  : useSliver = false,
        itemExtent = null,
        semanticChildCount = null,
        addAutomaticKeepAlives = true,
        addRepaintBoundaries = true,
        addSemanticIndexes = true,
        reorderableList = onReorder != null,
        super(key: key);

  const ListViewResourceWidget.sliver({
    Key? key,
    this.useSliver = true,
    required this.resource,
    required this.loadingTile,
    required this.tileMapper,
    this.refresh,
    this.separatorBuilder,
    this.emptyWidget,
    this.topWidgets = const [],
    this.errorWidget,
    this.addAutomaticKeepAlives = true,
    this.addRepaintBoundaries = true,
    this.addSemanticIndexes = true,
    this.loadingTileQuantity = 2,
  })  : padding = null,
        scrollDirection = Axis.vertical,
        reverse = false,
        controller = null,
        primary = null,
        physics = null,
        shrinkWrap = false,
        itemExtent = null,
        cacheExtent = null,
        semanticChildCount = null,
        dragStartBehavior = DragStartBehavior.start,
        keyboardDismissBehavior = ScrollViewKeyboardDismissBehavior.manual,
        restorationId = null,
        clipBehavior = Clip.hardEdge,
        reorderableList = false,
        onReorder = null,
        super(key: key);

  @override
  Widget build(BuildContext context) {
    Widget child;

    if (useSliver) {
      child = SliverList(
        delegate: SliverChildListDelegate.fixed(
          _generateList(
            resource,
          ),
          addAutomaticKeepAlives: addAutomaticKeepAlives,
          addRepaintBoundaries: addRepaintBoundaries,
          addSemanticIndexes: addSemanticIndexes,
        ),
      );
    } else if (reorderableList) {
      child = ReorderableListView(
        onReorder: onReorder ?? (a, b) {},
        shrinkWrap: shrinkWrap,
        padding: padding,
        scrollDirection: scrollDirection,
        physics: physics,
        scrollController: controller,
        cacheExtent: cacheExtent,
        reverse: reverse,
        keyboardDismissBehavior: keyboardDismissBehavior,
        dragStartBehavior: dragStartBehavior,
        restorationId: restorationId,
        clipBehavior: clipBehavior,
        primary: primary,
        header: topWidgets.length == 1
            ? topWidgets.first
            : Column(
                mainAxisSize: MainAxisSize.min,
                children: topWidgets,
              ),
        children: _generateList(resource),
      );
    } else {
      child = ListView(
        padding: padding,
        scrollDirection: scrollDirection,
        reverse: reverse,
        controller: controller,
        primary: primary,
        physics: physics,
        shrinkWrap: shrinkWrap,
        itemExtent: itemExtent,
        addAutomaticKeepAlives: addAutomaticKeepAlives,
        addRepaintBoundaries: addRepaintBoundaries,
        addSemanticIndexes: addSemanticIndexes,
        cacheExtent: cacheExtent,
        semanticChildCount: semanticChildCount,
        dragStartBehavior: dragStartBehavior,
        keyboardDismissBehavior: keyboardDismissBehavior,
        restorationId: restorationId,
        clipBehavior: clipBehavior,
        children: _generateList(
          resource,
        ),
      );
    }
    if (refresh != null && !useSliver) {
      return RefreshIndicator(
        onRefresh: refresh!,
        child: child,
      );
    }
    return child;
  }

  List<Widget> _generateList(
    Resource<List<T>> resource,
  ) {
    var listWidgets = <Widget>[];
    if (resource.data != null) {
      listWidgets = (resource.data ?? []).map<Widget>(tileMapper).toList();
      if (separatorBuilder != null) {
        listWidgets = listWidgets
            .asMap()
            .map((index, child) {
              return MapEntry(index, separatorBuilder!(index, child));
            })
            .values
            .toList();
      }
    }
    switch (resource.status) {
      case Status.loading:
        for (var i = 0; i < loadingTileQuantity; i++) {
          listWidgets.add(loadingTile);
        }
        break;
      case Status.success:
        if (emptyWidget != null &&
            (resource.data == null || (resource.data ?? []).isEmpty)) {
          listWidgets.add(emptyWidget ?? const SizedBox.shrink());
        }
        break;
      case Status.failed:
        listWidgets.add(
          errorWidget != null
              ? errorWidget!(resource.error ?? const AppException())
              : ErrorSnowWidget(
                  resource.message,
                  onTryAgain: refresh,
                  key: UniqueKey(),
                ),
        );
        break;
      default:
        break;
    }
    if (!reorderableList) {
      listWidgets.insertAll(0, topWidgets);
    }
    return listWidgets;
  }
}

==> flutter_snow_base-2.2.3.tar.gz: lib/src/widgets/list_view/list_view_resource_widget.dart <==
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:flutter_snow_base/flutter_snow_base.dart';
import 'package:flutter_snow_base/src/widgets/error_snow/error_snow_widget.dart';

class ListViewResourceWidget<T> extends StatelessWidget {
  final bool useSliver;
  final Resource<List<T>> resource;
  final Widget loadingTile;
  final Widget Function(T data) tileMapper;
  final Widget Function(int index, Widget tile)? separatorBuilder;
  final Widget? emptyWidget;
  final Future<void> Function()? refresh;
  final EdgeInsets? padding;
  final List<Widget> topWidgets;
  final Widget Function(AppException e)? errorWidget;
  final Axis scrollDirection;
  final bool reverse;
  final ScrollController? controller;
  final bool? primary;
  final ScrollPhysics? physics;
  final bool shrinkWrap;
  final bool addAutomaticKeepAlives;
  final bool addRepaintBoundaries;
  final bool addSemanticIndexes;
  final double? cacheExtent;
  final int? semanticChildCount;
  final DragStartBehavior dragStartBehavior;
  final ScrollViewKeyboardDismissBehavior keyboardDismissBehavior;
  final String? restorationId;
  final Clip clipBehavior;
  final double? itemExtent;
  final bool reorderableList;
  final ReorderCallback? onReorder;
  final int loadingTileQuantity;

  const ListViewResourceWidget({
    Key? key,
    this.useSliver = false,
    required this.resource,
    required this.loadingTile,
    required this.tileMapper,
    this.refresh,
    this.emptyWidget,
    this.padding,
    this.topWidgets = const [],
    this.errorWidget,
    this.scrollDirection = Axis.vertical,
    this.reverse = false,
    this.controller,
    this.primary,
    this.physics,
    this.shrinkWrap = true,
    this.itemExtent,
    this.addAutomaticKeepAlives = true,
    this.addRepaintBoundaries = true,
    this.addSemanticIndexes = true,
    this.cacheExtent,
    this.semanticChildCount,
    this.dragStartBehavior = DragStartBehavior.start,
    this.keyboardDismissBehavior = ScrollViewKeyboardDismissBehavior.manual,
    this.restorationId,
    this.clipBehavior = Clip.hardEdge,
    this.loadingTileQuantity = 2,
    this.separatorBuilder,
  })  : reorderableList = false,
        onReorder = null,
        super(key: key);

  const ListViewResourceWidget.reorderable({
    Key? key,
    required this.resource,
    required this.loadingTile,
    required this.tileMapper,
    required this.onReorder,
    this.refresh,
    this.emptyWidget,
    this.topWidgets = const [],
    this.errorWidget,
    this.padding,
    this.controller,
    this.scrollDirection = Axis.vertical,
    this.shrinkWrap = true,
    this.loadingTileQuantity = 2,
    this.separatorBuilder,
    this.physics,
    this.reverse = false,
    this.keyboardDismissBehavior = ScrollViewKeyboardDismissBehavior.manual,
    this.dragStartBehavior = DragStartBehavior.start,
    this.restorationId,
    this.cacheExtent,
    this.clipBehavior = Clip.hardEdge,
    this.primary,
  })  : useSliver = false,
        itemExtent = null,
        semanticChildCount = null,
        addAutomaticKeepAlives = true,
        addRepaintBoundaries = true,
        addSemanticIndexes = true,
        reorderableList = onReorder != null,
        super(key: key);

  const ListViewResourceWidget.sliver({
    Key? key,
    this.useSliver = true,
    required this.resource,
    required this.loadingTile,
    required this.tileMapper,
    this.refresh,
    this.separatorBuilder,
    this.emptyWidget,
    this.topWidgets = const [],
    this.errorWidget,
    this.addAutomaticKeepAlives = true,
    this.addRepaintBoundaries = true,
    this.addSemanticIndexes = true,
    this.loadingTileQuantity = 2,
  })  : padding = null,
        scrollDirection = Axis.vertical,
        reverse = false,
        controller = null,
        primary = null,
        physics = null,
        shrinkWrap = false,
        itemExtent = null,
        cacheExtent = null,
        semanticChildCount = null,
        dragStartBehavior = DragStartBehavior.start,
        keyboardDismissBehavior = ScrollViewKeyboardDismissBehavior.manual,
        restorationId = null,
        clipBehavior = Clip.hardEdge,
        reorderableList = false,
        onReorder = null,
        super(key: key);

  @override
  Widget build(BuildContext context) {
    Widget child;
    final list = _generateList(resource);

    if (useSliver) {
      child = SliverList(
        delegate: SliverChildListDelegate.fixed(
          list,
          addAutomaticKeepAlives: addAutomaticKeepAlives,
          addRepaintBoundaries: addRepaintBoundaries,
          addSemanticIndexes: addSemanticIndexes,
        ),
      );
    } else if (reorderableList) {
      child = ReorderableListView(
        onReorder: onReorder ?? (a, b) {},
        shrinkWrap: shrinkWrap,
        padding: padding,
        scrollDirection: scrollDirection,
        physics: physics,
        scrollController: controller,
        cacheExtent: cacheExtent,
        reverse: reverse,
        keyboardDismissBehavior: keyboardDismissBehavior,
        dragStartBehavior: dragStartBehavior,
        restorationId: restorationId,
        clipBehavior: clipBehavior,
        primary: primary,
        header: topWidgets.length == 1
            ? topWidgets.first
            : scrollDirection == Axis.vertical
                ? Column(
                    mainAxisSize: MainAxisSize.min,
                    children: topWidgets,
                  )
                : Row(
                    mainAxisSize: MainAxisSize.min,
                    children: topWidgets,
                  ),
        children: list,
      );
    } else {
      child = ListView(
        padding: padding,
        scrollDirection: scrollDirection,
        reverse: reverse,
        controller: controller,
        primary: primary,
        physics: physics,
        shrinkWrap: shrinkWrap,
        itemExtent: itemExtent,
        addAutomaticKeepAlives: addAutomaticKeepAlives,
        addRepaintBoundaries: addRepaintBoundaries,
        addSemanticIndexes: addSemanticIndexes,
        cacheExtent: cacheExtent,
        semanticChildCount: semanticChildCount,
        dragStartBehavior: dragStartBehavior,
        keyboardDismissBehavior: keyboardDismissBehavior,
        restorationId: restorationId,
        clipBehavior: clipBehavior,
        children: list,
      );
    }
    if (refresh != null && !useSliver) {
      return RefreshIndicator(
        onRefresh: refresh!,
        child: child,
      );
    }
    return child;
  }

  List<Widget> _generateList(
    Resource<List<T>> resource,
  ) {
    var listWidgets = <Widget>[];
    if (resource.data != null) {
      listWidgets = (resource.data ?? []).map<Widget>(tileMapper).toList();
      if (separatorBuilder != null) {
        listWidgets = listWidgets
            .asMap()
            .map((index, child) {
              return MapEntry(index, separatorBuilder!(index, child));
            })
            .values
            .toList();
      }
    }
    switch (resource.status) {
      case Status.loading:
        for (var i = 0; i < loadingTileQuantity; i++) {
          topWidgets.add(loadingTile);
        }
        break;
      case Status.success:
        if (emptyWidget != null &&
            (resource.data == null || (resource.data ?? []).isEmpty)) {
          listWidgets.add(emptyWidget ?? const SizedBox.shrink());
        }
        break;
      case Status.failed:
        listWidgets.add(
          errorWidget != null
              ? errorWidget!(resource.error ?? const AppException())
              : ErrorSnowWidget(
                  resource.message,
                  onTryAgain: refresh,
                  key: UniqueKey(),
                ),
        );
        break;
      default:
        break;
    }
    if (!reorderableList) {
      listWidgets.insertAll(0, topWidgets);
    }
    return listWidgets;
  }
}
